import React, { useState, useEffect, useReducer } from 'react';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function useFunction() {
  for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    args[_key2] = arguments[_key2];
  }

  var _useState2 = useState(),
      results = _useState2[0],
      setResults = _useState2[1];

  useEffect(function () {
    if (typeof args[0] === 'function') {
      var func = args.splice(0, 1);
      setResults(func[0].apply(func, args));
    } else {
      throw new Error('useFunction takes first argument as a function');
    }
  }, [JSON.stringify(args)]);
  return results;
}

var isBetween = function isBetween(min, max, num) {
  var states = num >= min && max >= num;
  return states;
};

function numToObj(item) {
  if (!isNaN(item)) {
    return {
      width: item,
      height: item
    };
  }

  return item;
}
function getPosition(row, column, dimension, rowTitleWidth, columnTitleHeight) {
  if (rowTitleWidth === void 0) {
    rowTitleWidth = 0;
  }

  if (columnTitleHeight === void 0) {
    columnTitleHeight = 0;
  }

  dimension = numToObj(dimension);
  return {
    left: rowTitleWidth + column * dimension.width,
    top: row * dimension.height + columnTitleHeight
  };
}
function getColumnCount(dimension, width, rowTitleWidth) {
  dimension = numToObj(dimension);
  return Math.floor((width - rowTitleWidth) / dimension.width);
}
function getRowCount(dimension, height) {
  dimension = numToObj(dimension);
  return Math.floor(height / dimension.height);
}
function resizeBars(bars, newLocation, resolver) {
  return bars.map(function (bar) {
    if (bar.editing) {
      var nBar = evaluatePosition(bar, newLocation.cell);

      if (typeof resolver === 'function') {
        nBar = resolver(nBar);
      }

      return nBar;
    }

    return bar;
  });
}
function finishEditingBars(bars) {
  return bars.map(function (bar) {
    if (bar.editing) {
      return _extends({}, bar, {
        editing: false
      });
    }

    return bar;
  });
}
function evaluatePosition(bar, newLocation) {
  if (bar.column > newLocation.column || bar.stick === 'right' && bar.length > 1) {
    bar.stick = 'right';
    var locationForRight = {
      row: bar.row,
      column: newLocation.column,
      length: bar.column - newLocation.column + bar.length
    };
    return _extends({}, bar, locationForRight);
  }

  bar.stick = 'left';
  var locationForLeft = {
    row: bar.row,
    column: bar.column,
    length: newLocation.column - bar.column + 1
  };
  return _extends({}, bar, locationForLeft);
}

function checkCollisions(bars) {
  var nBars = [].concat(bars);
  var editingBars = bars.filter(function (b) {
    return b.editing;
  });
  editingBars.forEach(function (bar) {
    var _reviewBars = reviewBars(nBars, bar),
        cBars = _reviewBars[0],
        eBar = _reviewBars[1];

    cBars.push(eBar);
    nBars = cBars;
  });
  return nBars;
}
function reviewBars(bars, eBar) {
  var editingBar = _extends({}, eBar);

  var oBars = bars.flatMap(function (b) {
    if (b.id === eBar.id) {
      return [];
    }

    if (editingBar.row === b.row) {
      var bStart = b.column + 1;
      var barStart = editingBar.column + 1;
      var bTotal = b.column + b.length;
      var editingBarTotal = editingBar.column + editingBar.length;

      if (isBetween(bStart, bTotal, barStart) || isBetween(bStart, bTotal, editingBarTotal) || isBetween(barStart, editingBarTotal, bTotal) || isBetween(barStart, editingBarTotal, bStart)) {
        var _collided = collided(b, editingBar),
            bar1 = _collided[0],
            bar2 = _collided[1];

        editingBar = bar2;
        return bar1;
      } else {
        var _removeCollision = removeCollision(b, editingBar),
            _bar = _removeCollision[0],
            _bar2 = _removeCollision[1];

        editingBar = _bar2;
        return _bar;
      }
    } else {
      var _removeCollision2 = removeCollision(b, editingBar),
          _bar3 = _removeCollision2[0],
          _bar4 = _removeCollision2[1];

      editingBar = _bar4;
      return _bar3;
    }
  });
  return [oBars, editingBar];
}
var collided = function collided(bar1, bar2) {
  bar1 = checkHasCollisionObject(bar1);
  bar1.collisions[bar2.id] = '';
  bar2 = checkHasCollisionObject(bar2);
  bar2.collisions[bar1.id] = '';
  return [bar1, bar2];
};
var checkHasCollisionObject = function checkHasCollisionObject(bar) {
  if (!bar.collisions) {
    bar.collisions = {};
  }

  return bar;
};
var removeCollision = function removeCollision(bar1, bar2) {
  bar1 = checkHasCollisionObject(bar1);
  delete bar1.collisions[bar2.id];
  bar2 = checkHasCollisionObject(bar2);
  delete bar2.collisions[bar1.id];
  return [bar1, bar2];
};

var actionTypes = {
  add: 'ADD',
  edit: 'EDIT',
  "delete": 'DELETE',
  setBars: 'SET_BARS',
  setIsEditing: 'SET_IS_EDITING'
};

function reserverReducer(state, action) {
  switch (action.type) {
    case actionTypes.setBars:
      {
        return _extends({}, state, {
          bars: action.payload
        });
      }

    case actionTypes.edit:
      {
        var nBars = [].concat(state.bars);
        var bIndex = nBars.findIndex(function (bar) {
          return action.payload.id === bar.id;
        });
        nBars[bIndex] = action.payload;
        return _extends({}, state, {
          bars: nBars
        });
      }

    case actionTypes.add:
      {
        var bars = [].concat(state.bars);
        bars.push(action.payload);
        return _extends({}, state, {
          bars: bars
        });
      }

    case actionTypes["delete"]:
      {
        var _nBars = [].concat(state.bars);

        var _bIndex = _nBars.findIndex(function (bar) {
          return action.payload.id === bar.id;
        });

        _nBars.splice(_bIndex, 1);

        return _extends({}, state, {
          bars: _nBars
        });
      }

    case actionTypes.setIsEditing:
      {
        return _extends({}, state, {
          isEditing: action.payload
        });
      }
  }
}

function useReserver(reducer, initialState) {
  var _useReducer = useReducer(reducer, {
    bars: initialState,
    isEditing: false
  }),
      _useReducer$ = _useReducer[0],
      bars = _useReducer$.bars,
      isEditing = _useReducer$.isEditing,
      dispatch = _useReducer[1];

  var addBar = function addBar(props) {
    return dispatch({
      payload: props,
      type: actionTypes.add
    });
  };

  var editBar = function editBar(props) {
    return dispatch({
      payload: props,
      type: actionTypes.edit
    });
  };

  var deleteBar = function deleteBar(props) {
    return dispatch({
      payload: props,
      type: actionTypes["delete"]
    });
  };

  var setBars = function setBars(props) {
    return dispatch({
      payload: props,
      type: actionTypes.setBars
    });
  };

  var setIsEditing = function setIsEditing(props) {
    return dispatch({
      payload: props,
      type: actionTypes.setIsEditing
    });
  };

  return {
    isEditing: isEditing,
    setIsEditing: setIsEditing,
    bars: bars,
    addBar: addBar,
    editBar: editBar,
    deleteBar: deleteBar,
    setBars: setBars
  };
}

function isObject(item) {
  return typeof item === 'object' && !Array.isArray(item) && item !== null;
}

function generateKey(param, level, maxLevel) {
  if (level === void 0) {
    level = 0;
  }

  if (maxLevel === void 0) {
    maxLevel = 4;
  }

  if (level === maxLevel) return;

  if (Array.isArray(param)) {
    return param.map(function (elem) {
      return generateKey(elem, level + 1);
    });
  }

  if (isObject(param)) {
    return Object.keys(param).map(function (key) {
      return key + ' : ' + generateKey(param[key], level + 1);
    });
  }

  if (typeof param === 'function') {
    return param.name + 'function';
  }

  if (typeof param === 'string' || typeof param === 'number') {
    return param;
  }

  return JSON.stringify(param);
}

function useFuncOrObj() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  var _useState = useState({}),
      results = _useState[0],
      setResults = _useState[1];

  useEffect(function () {
    if (isObject(args[0])) {
      setResults(args[0]);
    } else if (typeof args[0] === 'function') {
      var func = args.splice(0, 1);
      setResults(func[0].apply(func, args));
    }
  }, [JSON.stringify(generateKey(args))]);
  return results;
}

function getContent(index, length, content, firstContent, lastContent) {
  if (index === 0) {
    return firstContent || content[index] || /*#__PURE__*/React.createElement("div", null);
  } else if (length - 1 === index) {
    return lastContent || content[length - 1] || /*#__PURE__*/React.createElement("div", null);
  }

  return content[index] || /*#__PURE__*/React.createElement("div", null);
}

function Bar(props) {
  var content = useFuncOrObj(props.content, props.length);
  return /*#__PURE__*/React.createElement("div", {
    id: props.id,
    role: "listitem",
    onDragStart: function onDragStart(e) {
      props.onDragStart(e, props);
    },
    onDragEnd: function onDragEnd(e) {
      props.onDragEnd(e, props);
    },
    onClick: function onClick(e) {
      props.onClick(e, props);
    },
    onMouseOver: function onMouseOver(e) {
      props.onMouseOver(e, props);
    },
    onContextMenu: function onContextMenu(e) {
      props.onContextMenu(e, props);
    },
    onMouseEnter: function onMouseEnter(e) {
      props.onMouseEnter(e, props);
    },
    onMouseLeave: function onMouseLeave(e) {
      props.onMouseLeave(e, props);
    },
    onMouseMove: function onMouseMove(e) {
      props.onMouseMove(e, props);
    },
    onMouseDown: function onMouseDown(e) {
      props.onMouseDown(e, props);
    },
    onMouseUp: function onMouseUp(e) {
      props.onMouseUp(e, props);
    },
    draggable: props.draggable,
    style: _extends({}, props.style, {
      pointerEvents: props.style.pointerEvents || 'none',
      background: props.style.background || '#0E6BA8',
      display: props.style.display || 'flex',
      position: props.style.position || 'absolute',
      zIndex: props.style.zIndex || '100'
    }),
    className: props.className
  }, [].concat(Array(props.length)).map(function (notUsed, i) {
    var processedContent = getContent(i, props.length, content, props.firstContent, props.lastContent);
    var style = Object.assign({
      width: props.dimension.width,
      height: props.dimension.height,
      pointerEvents: props.style.pointerEvents || 'none'
    }, processedContent.props.style || {});
    return /*#__PURE__*/React.createElement(React.Fragment, {
      key: i
    }, React.cloneElement(processedContent, _extends({}, processedContent.props, {
      style: style
    }), processedContent.props.children));
  }), props.children);
}
Bar.defaultProps = {
  style: {},
  dimension: {
    width: 20,
    height: 20
  },
  onClick: function onClick() {},
  onMouseOver: function onMouseOver() {},
  onDragStart: function onDragStart() {},
  onDragEnd: function onDragEnd() {},
  onContextMenu: function onContextMenu() {},
  onMouseDown: function onMouseDown() {},
  onMouseUp: function onMouseUp() {},
  onMouseEnter: function onMouseEnter() {},
  onMouseLeave: function onMouseLeave() {},
  onMouseMove: function onMouseMove() {},
  length: 1,
  content: {}
};

function Cell(props) {
  return /*#__PURE__*/React.createElement("div", {
    role: "gridcell",
    "aria-colindex": props.column,
    onDragStart: function onDragStart(e) {
      e.preventDefault();
    },
    onDragOver: function onDragOver(e) {
      e.preventDefault();
      props.onDragOver({
        cell: {
          row: props.row,
          column: props.column
        }
      }, e);
    },
    className: props.className,
    onMouseOver: function onMouseOver(e) {
      props.onMouseOver({
        dimension: props.dimension,
        cell: {
          row: props.row,
          column: props.column
        }
      }, e);
    },
    onMouseEnter: function onMouseEnter(e) {
      props.onMouseEnter({
        dimension: props.dimension,
        cell: {
          row: props.row,
          column: props.column
        }
      }, e);
    },
    onMouseDown: function onMouseDown(e) {
      props.onMouseDown({
        dimension: props.dimension,
        cell: {
          row: props.row,
          column: props.column
        }
      }, e);
    },
    onMouseUp: function onMouseUp(e) {
      props.onMouseUp({
        cell: {
          row: props.row,
          column: props.column
        }
      }, e);
    },
    onDrop: function onDrop(e) {
      props.onDrop({
        cell: {
          row: props.row,
          column: props.column
        }
      }, e);
    },
    style: _extends({
      overflow: 'hidden',
      width: props.dimension.width,
      height: props.dimension.height
    }, props.style)
  }, props.children);
}
Cell.defaultProps = {
  onMouseEnter: function onMouseEnter() {},
  onMouseDown: function onMouseDown() {},
  onMouseUp: function onMouseUp() {},
  onMouseOver: function onMouseOver() {},
  onDrop: function onDrop() {}
};

function Head(props) {
  return /*#__PURE__*/React.createElement("div", {
    role: "columnheader",
    className: props.rowTitleClassName,
    style: {
      display: props.isVisible ? 'flex' : 'none',
      height: props.dimension.height
    }
  }, props.dir === 'ltr' && /*#__PURE__*/React.createElement(Cell, {
    dimension: {
      height: props.dimension.height,
      width: props.rowTitleWidth
    },
    className: props.cantonClassName
  }, props.canton), props.columnTitles.map(function (headitem, i) {
    return /*#__PURE__*/React.createElement(Cell, {
      "aria-colindex": i,
      key: i,
      onMouseOver: props.onMouseOverCell,
      dimension: props.dimension,
      column: i,
      row: -1,
      className: props.columnTitleClassName
    }, headitem);
  }), props.dir === 'rtl' && /*#__PURE__*/React.createElement(Cell, {
    dimension: {
      height: props.dimension.height,
      width: props.rowTitleWidth
    },
    className: props.cantonClassName
  }, props.canton));
}
Head.defaultProps = {
  onMouseOverCell: function onMouseOverCell() {}
};

function Peg(props) {
  var style = {
    userSelect: 'none',
    pointerEvents: 'none',
    alignItems: 'center',
    height: '100%',
    width: '100%',
    display: 'flex',
    fontSize: '10px',
    background: 'green'
  };
  return /*#__PURE__*/React.createElement("div", {
    className: props.className,
    style: Object.assign(style, props.style)
  }, props.children);
}

function Tag(props) {
  var style = {
    textAlign: 'center',
    position: 'absolute',
    whiteSpace: 'nowrap',
    top: '0px',
    overflow: 'hidden',
    userSelect: 'none',
    lineHeight: 1,
    height: '100%'
  };
  return /*#__PURE__*/React.createElement("div", {
    style: Object.assign(style, props.style),
    className: props.className
  }, props.children);
}

function makeID(length) {
  if (length === void 0) {
    length = 15;
  }

  var text = '';
  var possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';

  for (var i = 0; i < length; i++) {
    text += possible.charAt(Math.floor(Math.random() * possible.length));
  }

  return text;
}

function createBar(dimension, startLocation, props) {
  return _extends({
    id: makeID(),
    length: 1,
    dimension: dimension,
    editing: true
  }, startLocation, props);
}

function useDimension(item) {
  var _useState = useState({
    width: 0,
    height: 0
  }),
      dimension = _useState[0],
      setDimension = _useState[1];

  useEffect(function () {
    if (!isNaN(item)) {
      setDimension({
        width: item,
        height: item
      });
    } else if (isObject(item)) {
      setDimension(item);
    } else {
      setDimension({
        width: -1,
        height: -1
      });
    }
  }, [JSON.stringify(item)]);
  return dimension;
}

function useFunctionObjectArray() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  var _useState = useState([]),
      results = _useState[0],
      setResults = _useState[1];

  useEffect(function () {
    if (isObject(args[0])) {
      var func = args[0].func;

      if (typeof func === 'function') {
        args.splice(0, 1);
        setResults(func.apply(void 0, args));
      }
    } else if (typeof args[0] === 'function') {
      var _func = args.splice(0, 1);

      setResults(_func[0].apply(_func, args));
    } else if (Array.isArray(args[0])) {
      setResults(args[0]);
    }
  }, [JSON.stringify(args)]);
  return results;
}

var Reserver = React.forwardRef(function (props, ref) {
  var rowCount = useFunction(getRowCount, props.dimension, props.height);
  var columnCount = useFunction(getColumnCount, props.dimension, props.width, props.rowTitleWidth);
  var rowTitles = useFunctionObjectArray(props.rowTitles, rowCount);
  var columnTitles = useFunctionObjectArray(props.columnTitles, columnCount);
  var content = useFuncOrObj(props.content, columnCount, rowCount);
  var dimension = useDimension(props.dimension);
  return /*#__PURE__*/React.createElement("div", {
    ref: ref,
    id: props.id,
    className: props.className,
    role: "grid",
    onMouseLeave: props.mouseLeaveGrid,
    onMouseMove: props.mouseMoveGrid,
    style: _extends({}, props.style, {
      position: 'relative'
    })
  }, /*#__PURE__*/React.createElement(Head, {
    columnTitles: columnTitles,
    columnCount: columnCount,
    height: props.columnTitleHeight,
    rowTitleWidth: props.rowTitleWidth,
    dimension: dimension,
    isVisible: columnTitles.length > 0,
    columnTitleClassName: props.columnTitleClassName,
    dir: props.dir,
    onMouseOverCell: props.mouseOverCellHead,
    cantonClassName: props.cantonClassName
  }), [].concat(Array(rowCount)).map(function (x, r) {
    return /*#__PURE__*/React.createElement("div", {
      role: "rowgroup",
      key: r,
      style: {
        height: dimension.height,
        display: 'flex'
      }
    }, props.dir === 'ltr' && /*#__PURE__*/React.createElement(Cell, {
      style: {
        display: rowTitles.length > 0 ? 'initial' : 'none'
      },
      dimension: {
        height: dimension.height,
        width: props.rowTitleWidth
      },
      className: props.rowTitleClassName
    }, rowTitles[r]), [].concat(Array(columnCount)).map(function (x, c) {
      return /*#__PURE__*/React.createElement(Cell, {
        key: "r" + r + "c" + c,
        onMouseDown: props.mouseDownCell,
        onMouseEnter: props.mouseEnterCell,
        onMouseUp: props.mouseUpCell,
        onDrop: props.mouseDropCell,
        onDragOver: props.mouseDragOverCell,
        dimension: dimension,
        className: props.cellClassName,
        column: c,
        row: r
      }, content["r" + r + "c" + c]);
    }), props.dir === 'rtl' && /*#__PURE__*/React.createElement(Cell, {
      style: {
        display: rowTitles.length > 0 ? 'initial' : 'none'
      },
      dimension: {
        height: dimension.height,
        width: props.rowTitleWidth
      },
      className: props.rowTitleClassName
    }, rowTitles[r]));
  }), /*#__PURE__*/React.createElement("div", {
    role: "list"
  }, typeof props.children === 'function' && props.children({
    rowCount: rowCount,
    columnCount: columnCount,
    rowTitleWidth: props.rowTitleWidth,
    dimension: dimension,
    columnTitleHeight: columnTitles.length > 0 ? props.columnTitleHeight > 0 ? props.columnTitleHeight : dimension.height : 0
  }), Array.isArray(props.children) && props.children));
});
Reserver.defaultProps = {
  columnTitles: [],
  rowTitles: [],
  content: {},
  dimension: 20,
  width: 500,
  height: 500,
  rowTitleWidth: 0,
  columnTitleHeight: 0,
  dir: 'ltr',
  mouseEnterCell: function mouseEnterCell() {},
  mouseDownCell: function mouseDownCell() {},
  mouseUpCell: function mouseUpCell() {},
  mouseDragOverCell: function mouseDragOverCell() {},
  mouseDropCell: function mouseDropCell() {},
  mouseLeaveGrid: function mouseLeaveGrid() {},
  mouseMoveGrid: function mouseMoveGrid() {}
};

export default Reserver;
export { Bar, Peg, Reserver, Tag, actionTypes, checkCollisions, createBar, evaluatePosition, finishEditingBars, getPosition, reserverReducer, resizeBars, useReserver };
//# sourceMappingURL=index.modern.js.map
